@@ -51,7 +51,8 @@ class MetricBuffer:
     self.aggregation_frequency = int(frequency)
     self.aggregation_func = func
     self.compute_task = LoopingCall(self.compute_value)
-    self.compute_task.start(settings['WRITE_BACK_FREQUENCY'] or frequency, now=False)
+    compute_frequency = min(settings['WRITE_BACK_FREQUENCY'], frequency) or frequency
+    self.compute_task.start(compute_frequency, now=False)
     self.configured = True
 
   def compute_value(self):
@@ -63,8 +64,8 @@ class MetricBuffer:
       if buffer.active:
         value = self.aggregation_func(buffer.values)
         datapoint = (buffer.interval, value)
-        events.metricGenerated(self.metric_path, datapoint)
-        instrumentation.increment('aggregateDatapointsSent')
+        state.events.metricGenerated(self.metric_path, datapoint)
+        state.instrumentation.increment('aggregateDatapointsSent')
         buffer.mark_inactive()
 
       if buffer.interval < age_threshold:
@@ -103,4 +104,4 @@ class IntervalBuffer:
 BufferManager = BufferManager()
 
 # Avoid import circularity
-from carbon import events, state, instrumentation
+from carbon import state
